@@ -1,20 +1,14 @@
 #!/usr/bin/env python
 # -*- coding: utf8 -*-
 
-#
-# LICENSE:
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU Affero General Public License version 3 (AGPL)
-# as published by the Free Software Foundation.
-# (c) 2019 caregraf
-#
+# (c) 2017-2020 caregraf
 
 import os
 import json
 from datetime import datetime, timedelta
 import pytz
 import re
-from collections import defaultdict, OrderedDict
+from collections import defaultdict
 import math
 import logging
 from logging.handlers import RotatingFileHandler
@@ -48,7 +42,7 @@ TODO:
 class FMQLReplyStore:
 
     # useZip only applies to EMPTY or newly created
-    def __init__(self, cacheLocation, isOrdered=True, useZip=False, makeDir=True):
+    def __init__(self, cacheLocation, useZip=False, makeDir=True):
     
         """
         Test for both type of cache (ZIP or RAW) and if good or not (BAD_)
@@ -58,12 +52,12 @@ class FMQLReplyStore:
                 return "EMPTY"
             zfs = set(zf for zf in os.listdir(cacheLocation) if re.search(r'\.zip', zf))
             if len(zfs):
-                if sum(1 for zf in zfs if not re.match(r'[\d\_]+\-[\d\.]+\.zip$', zf)):
+                if sum(1 for zf in zfs if not re.match(r'[\d\_]+(\-S)?\-[\d\.]+\.zip$', zf)):
                     return "BAD_ZIP"
                 return "ZIP"
             jfs = set(jf for jf in os.listdir(cacheLocation) if re.search(r'\.json', jf))
             if len(jfs):
-                if sum(1 for bjf in jfs if not re.match(r'[\d\_]+-[\d\.]+\.json$', bjf)):
+                if sum(1 for bjf in jfs if not re.match(r'[\d\_]+(\-S)?\-[\d\.]+\.json$', bjf)):
                     return "BAD_RAW"
                 return "RAW"
             return "BAD_NOTEMPTY"
@@ -84,13 +78,15 @@ class FMQLReplyStore:
             else:
                 self.__isZip = True if cType == "ZIP" else False 
         self.__cacheLocation = cacheLocation if re.search(r'\/$', cacheLocation) else cacheLocation + "/"
-        self.__isOrdered = isOrdered
         
     def usesZip(self):
         return self.__isZip
 
     def availableTypes(self):
         return set(f.split("-")[0] for f in self.replyFiles())
+
+    def stoppedTypes(self):
+        return set(f.split("-")[0] for f in self.replyFiles() if re.search(r'\-S\-', f))
         
     """
     From SELECT TYPES - expected to be in {cacheLocation}/Schema
@@ -237,23 +233,23 @@ class FMQLReplyStore:
                 jfl = re.sub(r'zip$', 'json', fl)
                 zfName = "{}/{}".format(self.__cacheLocation, fl)
                 zf = zipfile.ZipFile(zfName, "r")   
-                lastReply = json.loads(zf.read(jfl), object_pairs_hook=OrderedDict) if self.__isOrdered else json.loads(zf.read(jfl))
+                lastReply = json.loads(zf.read(jfl))
                 return lastReply
             else:
-                return json.load(open(self.__cacheLocation + fl), object_pairs_hook=OrderedDict) if self.__isOrdered else json.load(open(self.__cacheLocation + fl))
+                return json.load(open(self.__cacheLocation + fl))
         return None 
 
     def replyFilesOfType(self, replyType):
         filesSoFar = sorted([f for f in self.replyFiles() if re.match(replyType + "\-", f)], key=lambda x: float(re.sub(r'\.(json|zip)$', '', x.split("-")[-1])))
         return filesSoFar
         
-    def loadReplyFromFile(self, fl, isOrdered=False):
+    def loadReplyFromFile(self, fl):
         if re.search(r'\.zip$', fl):
             zf = zipfile.ZipFile(self.__cacheLocation + fl, "r")
             jfl = re.sub(r'zip$', 'json', fl)
-            flJSON = json.loads(zf.read(jfl), object_pairs_hook=OrderedDict) if isOrdered else json.loads(zf.read(jfl))
+            flJSON = json.loads(zf.read(jfl))
         else:
-            flJSON = json.load(open(self.__cacheLocation + "/" + fl), object_pairs_hook=OrderedDict) if isOrdered else json.load(open(self.__cacheLocation + "/" + fl))
+            flJSON = json.load(open(self.__cacheLocation + "/" + fl))
         return flJSON
         
     def totalReplies(self):
@@ -264,11 +260,11 @@ class FMQLReplyStore:
     """
     def flush(self, reply):     
         # Expect FLOAT or INTEGER IEN so that sorting will work
-        if not re.match(r'[\d\.]+$', reply["fmql"]["AFTERIEN"]):
+        if not re.match(r'[\d\.]+$', str(reply["fmql"]["AFTERIEN"])):
             raise Exception("Ordering and processing of Replies in Cache rely on integer or float IENs - not {}".format(reply["fmql"]["AFTERIEN"]))
         if len(reply["results"]) == 0:
             raise Exception("Not allowed flush an empty reply - certain cache routines expect at least one result in each cached reply")
-        stoppedDesignation = "-S" if "stopped" in reply else ""
+        stoppedDesignation = "-S" if ("stopped" in reply or "fmqlMaxStopped" in reply) else ""
         if self.__isZip:
             fl = "{}{}-{}.{}".format(reply["fmql"]["TYPE"], stoppedDesignation, reply["fmql"]["AFTERIEN"], "zip")    
             zfName = "{}/{}".format(self.__cacheLocation, fl)
@@ -303,9 +299,8 @@ class FMQLReplyStore:
     
         class FMQLReplyIterator:
         
-            def __init__(self, cacheLocation, onlyTypes, excludeRepliesFilter, startAtReply, isOrdered, isZip):
+            def __init__(self, cacheLocation, onlyTypes, excludeRepliesFilter, startAtReply, isZip):
                 
-                self.__isOrdered = isOrdered
                 self.__cacheLocation = cacheLocation
                 self.__excludeRepliesFilter = excludeRepliesFilter 
                 self.__startAtReply = startAtReply
@@ -330,11 +325,16 @@ class FMQLReplyStore:
                     
             def currentReplyFile(self):
                 return self.__currentReplyFile
-                                        
-            def __next__(self):
+                        
+            """
+            TODO: full iterator
+            - startAtReply could be passed to iter optionally 
+            - next takes over what iter does but must raise StopIteration() 
+            """                
+            def __iter__(self):
         
                 for typ in sorted(self.__flsByTyp, key=lambda x: float(re.sub("_", ".", x))):
-                
+
                     for fl in sorted(self.__flsByTyp[typ], key=lambda x: float(re.sub(r'\.(json|zip)$', '', x.split("-")[-1]))):
                                         
                         # Usually only used with one type being iterated
@@ -355,15 +355,15 @@ class FMQLReplyStore:
                             except:
                                 raise Exception("Can't handle bad reply ZIP file {}".format(fl))
                             jfl = re.sub(r'zip$', 'json', fl)
-                            flJSON = json.loads(zf.read(jfl), object_pairs_hook=OrderedDict) if self.__isOrdered else json.loads(zf.read(jfl))
+                            flJSON = json.loads(zf.read(jfl))
                         else:
-                            flJSON = json.load(open(self.__cacheLocation + "/" + fl), object_pairs_hook=OrderedDict) if self.__isOrdered else json.load(open(self.__cacheLocation + "/" + fl))            
+                            flJSON = json.load(open(self.__cacheLocation + "/" + fl))            
                     
                         yield flJSON
               
         if startAtReply != "" and not (onlyTypes and len(onlyTypes) == 1):
             raise Exception("Cannot have iterate with 'startAtReply' if don't specify one and only one type")      
-        return FMQLReplyIterator(self.__cacheLocation, onlyTypes, excludeRepliesFilter, startAtReply, self.__isOrdered, self.__isZip)
+        return FMQLReplyIterator(self.__cacheLocation, onlyTypes, excludeRepliesFilter, startAtReply, self.__isZip)
         
 """
 Simple resource iterator built over the FMQLReplyStore's iterator
@@ -372,16 +372,16 @@ Note on memory use:
 
     import psutil
     process = psutil.Process(os.getpid())
-    print "Memory Use End", process.memory_info().rss, process.memory_percent()
+    print("Memory Use End", process.memory_info().rss, process.memory_percent())
     
 can see 2.7% -> 3.5% just doing a no op iteration of a large file. GC has its own mind.
 Using ORDERED is both slower and more memory intensive (6.5%).
 """
 class FilteredResultIterator:
     
-    def __init__(self, replyLocation, typId, filt=None, isOrdered=True, startAtReply=""):
+    def __init__(self, replyLocation, typId, filt=None, startAtReply=""):
     
-        fmqlReplyStore = FMQLReplyStore(replyLocation, isOrdered=isOrdered)        
+        fmqlReplyStore = FMQLReplyStore(replyLocation)        
         self.__replyIter = fmqlReplyStore.iterator(onlyTypes=[typId], startAtReply=startAtReply)
         self.__filt = filt
             
@@ -392,7 +392,7 @@ class FilteredResultIterator:
         return self._numberFiltered
             
     def __iter__(self):
-        for reply in next(self.__replyIter):
+        for reply in self.__replyIter:
             for resource in reply["results"]:
                 if self.__filt and not self.__filt(resource):
                     self._numberFiltered += 1
@@ -429,6 +429,11 @@ To make FMQLIF:
                             or
         fmqlIF = FMQLBrokerIF(hostname, port, access, verify, osehraVISTA=False)
 
+Returns:
+    COMPLETE
+    HITMAX
+    NO_SUCH_FILE
+    EMPTY_FILE
 """                
 def cacheFileToStore(fmqlIF, fmqlReplyStore, fileType, limit=500, cstop=1000, maxNumber=-1):
 
@@ -448,8 +453,15 @@ def cacheFileToStore(fmqlIF, fmqlReplyStore, fileType, limit=500, cstop=1000, ma
         lastLimit = int(lastReply["fmql"]["LIMIT"])
         if lastLimit > len(lastReply["results"]):
             logging.info("Got all of {} already - moving on".format(fileType))
-            return True
-        afterien = lastReply["results"][-1]["uri"]["value"].split("-")[1]
+            return "COMPLETE"
+        if maxNumber != -1:
+            repliesSoFar = fmqlReplyStore.replyFilesOfType(fileType)
+            cachedSoFar = len(repliesSoFar) * lastLimit
+            if cachedSoFar > maxNumber:
+                logging.info("Cached so far of {} already exceeds max number of {} - moving on".format(cachedSoFar, maxNumber))
+                return "HITMAX"
+        lastResult = lastReply["results"][-1]
+        afterien = (lastResult["_id"] if "_id" in lastResult else lastResult["uri"]["value"]).split("-")[1]
         numberOfTypeCachedOverall = lastLimit * len(fmqlReplyStore.replyFilesOfType(fileType))
         logging.info("Filetype {}, Cached {:,} already, some left to get. Restarting with AFTERIEN {}".format(fileType, numberOfTypeCachedOverall, afterien))
 
@@ -469,21 +481,28 @@ def cacheFileToStore(fmqlIF, fmqlReplyStore, fileType, limit=500, cstop=1000, ma
         queryStart = datetime.now()
         jreply = fmqlIF.invokeQuery(query)
         if "error" in jreply:
+            # Shouldn't happen for Cacher as it walks Schema but for some VistAs (FOIA)
+            # definitions may be there though files are not?
+            if jreply["error"] == "No such file": # will treat like a "Zero file"
+                if queryNo != 1:
+                    raise Exception("Expect 'no such file' error with first query")
+                logging.info("Bailing on {} query as no such file".format(fileType))
+                return "NO_SUCH_FILE"
             logging.error("Received error reply: {}".format(jreply["error"]))
             raise Exception(jreply["error"])
 
         # Special case - first call (afterien=0) and no results => don't cache
         # ... REM: doing TOPONLY in case POPONLY is wrong. Means more queries but safer.
-        # Ex for CHCS Synth: [u'3_081', u'66', u'52'] not in POP but have "data"
+        # Ex for CHCS Synth: ['3_081', '66', '52'] not in POP but have "data"
         # ... REM: afterien != 0 and no results then still cache as it means the case of
         # second to last LIMITED query filled up and then the last just returns none.
         # Note: alt is do a COUNT and then DESCRIBE if need be but largely the same cost.
         if afterien == 0 and len(jreply["results"]) == 0:
             logging.info("Empty {} - 0 replies afterien 0".format(fileType))
-            return False 
+            return "EMPTY_FILE" 
 
-        # Don't return False (<=> empty file) - but no results left ... happens in rare case of LIMIT coinciding with end
-        # of previous reply
+        # Don't return False (<=> empty file) - but no results left ... happens in rare 
+        # case of LIMIT coinciding with end of previous reply
         if len(jreply["results"]) == 0:
             break
 
@@ -491,14 +510,14 @@ def cacheFileToStore(fmqlIF, fmqlReplyStore, fileType, limit=500, cstop=1000, ma
         jreply["queryTook"] = queryTook # time taken (queryCached time is set in store)
         # note time as UTC (can change to local times in a display)
         jreply["queryCached"] = datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ")
-        if "stopped" in jreply:
+        if "fmqlMaxStopped" in jreply or "stopped" in jreply:
             stopped = True
         try:
             fmqlReplyStore.flush(jreply)
         except:
             logging.error("Failed to serialize {}/{} - backing out".format(fileType, afterien))
             raise
-        logging.info("Queried and flushed {} afterien {} in {} with {} resources, any stopped {}".format(fileType, str(afterien), queryTook, len(jreply["results"]), "TRUE" if "stopped" in jreply else "FALSE"))
+        logging.info("Queried and flushed {} afterien {} in {} with {} resources, any stopped {}".format(fileType, str(afterien), queryTook, len(jreply["results"]), "**TRUE**" if stopped else "FALSE"))
         
         numberOfTypeCached += len(jreply["results"])
         numberOfTypeCachedOverall += len(jreply["results"])
@@ -510,15 +529,14 @@ def cacheFileToStore(fmqlIF, fmqlReplyStore, fileType, limit=500, cstop=1000, ma
             logging.debug("Breaking as got or exceeded maximum requested - {} - for {}".format(maxNumber, fileType))
             break
         lastResult = jreply["results"][-1]
-        lastId = lastResult["id"] if "id" in lastResult else lastResult["uri"]["value"]
-        afterien = lastId.split("-")[1]
+        afterien = (lastResult["_id"] if "_id" in lastResult else lastResult["uri"]["value"]).split("-")[1]
         if (queryNo % 100) == 0:
             logging.debug("So far this has taken {}".format(datetime.now() - start))
 
     noCachedMU = "{:,}".format(numberOfTypeCached) if numberOfTypeCachedOverall == numberOfTypeCached else "{:,}/{:,}".format(numberOfTypeCached, numberOfTypeCachedOverall)
     logging.info("Finished - cached {}, stopped {}, took {}".format(noCachedMU, "TRUE" if stopped else "FALSE", datetime.now() - start))
     
-    return True
+    return "COMPLETE"
    
 """
 Inside [cacheLocation], will zip all replies of type [fileType] AND remove the unzipped replies. Allows retroactive zipping
@@ -572,11 +590,8 @@ def convert_size(size_bytes):
    return "%s %s" % (s, size_name[i])
    
 """
-TODO: in v2er report stops and stop % at end so can redo quickly. Ditto for cacher
-proper (needs FMQL cha
-
-NOTE: stopped leads to 'fmqlStopped' property in record entry. REM: stopped is prelude
-to flipping or caught in health check and leads to rerun. For now, just note it too.
+Deprecated: older V2 from V1 FMQL. V3 FMQL now largely embeds the form created here. Here for old dataset transformation
+leveraging fmqlV2er
 """
 class V1RecordToV2:
 
@@ -598,7 +613,6 @@ class V1RecordToV2:
     
         typV2 = record["uri"]["label"].split("/")[0] + "-" + record["uri"]["value"].split("-")[0]
 
-        # TODO: actually TYPELABEL from reply has correct value (as would schema? ... make schema backed?)
         TYP_MAP = {
             "STATION NUMBER TIME SENSITIVE-389_9": "STATION NUMBER (TIME SENSITIVE)-389_9",
             "ICD OPERATION_PROCEDURE-80_1": "ICD OPERATION/PROCEDURE-80_1",
@@ -626,11 +640,14 @@ class V1RecordToV2:
     def __reframe(self, record, isMultiple=False):
 
         if isMultiple == False:
-            nrecord = OrderedDict([(self.__idProp, record["uri"]["value"]), ("type", self.__newType(record))])
+            nrecord = {
+                self.__idProp: record["uri"]["value"],
+                "type": self.__newType(record)
+            }
             self.__topRecord = nrecord # for ref back up of cnodes
         else:
             ien = record["uri"]["value"].split("-")[1].split("_")[0]
-            nrecord = OrderedDict([("ien", ien)])
+            nrecord = {"ien", ien}
             
         """
         Issue: until uses VistA time zone settings, FMQL "Z" (nominally UTC time)
@@ -669,15 +686,16 @@ class V1RecordToV2:
                         fmCount = int(propValue["fmCount"])
                     except:
                         fmCount = -1
+                    # Note: missing subTypeId in V3 as not available
                     if "fmqlStopped" not in nrecord:
                         nrecord["fmqlStopped"] = {}
-                        if "fmqlHasStops" not in self.__topRecord:
-                            self.__topRecord["fmqlHasStops"] = -1
+                        if "fmqlMaxStopped" not in self.__topRecord:
+                            self.__topRecord["fmqlMaxStopped"] = -1
                     # v1.3 added fmCount but can be BAD ie/ <= REPLY CSTOP
-                    nrecord["fmqlStopped"][nprop] = fmCount
+                    nrecord["fmqlStopped"][nprop] = {"count": fmCount}
                     # Ensure top record has maximum list size
-                    if self.__topRecord["fmqlHasStops"] < fmCount:
-                        self.__topRecord["fmqlHasStops"] = fmCount
+                    if self.__topRecord["fmqlMaxStopped"] < fmCount:
+                        self.__topRecord["fmqlMaxStopped"] = fmCount
                     logging.info("Stopped property {} of {}, count {}".format(prop, record["uri"]["value"], fmCount))
                     continue
                 nrecord[nprop] = []
@@ -688,9 +706,9 @@ class V1RecordToV2:
             # Note: no time zone proper yet -- leaving for V2 FMQL
             if propValue["fmType"] == "1": # datetime
                 if re.search(r'T00:00:00Z$', propValue["value"]):
-                    nvalue = OrderedDict([("value", propValue["value"].split("T")[0] ), ("type", "xsd:date")])
+                    nvalue = {"value": propValue["value"].split("T")[0], "type": "xsd:date"}
                 else:
-                    nvalue = OrderedDict([("value", fix24DateTime(propValue["value"])), ("type", "xsd:dateTime")])
+                    nvalue = {"value": fix24DateTime(propValue["value"]), "type": "xsd:dateTime"}
             elif propValue["fmType"] == "2": # treat numeric as literal just in case
                 nvalue = propValue["value"]
             # or could just 'ivalue' in record[prop]
@@ -698,12 +716,12 @@ class V1RecordToV2:
                 nvalue = "{}:{}".format(propValue["ivalue"], propValue["value"])
             elif propValue["fmType"] == "4": # literal
                 nvalue = propValue["value"]
-            # 'datatype': u'http://www.w3.org/1999/02/22-rdf-syntax-ns#XMLLiteral too
+            # 'datatype': 'http://www.w3.org/1999/02/22-rdf-syntax-ns#XMLLiteral too
             elif propValue["fmType"] == "5": 
                 nvalue = re.sub(r'\r', '\n', propValue["value"])
             # 6: COMPUTED
             elif propValue["fmType"] in ["7", "8"]:
-                nvalue = OrderedDict([("id", propValue["value"]), ("label", propValue["label"].split("/")[1])])
+                nvalue = {"id": propValue["value"], "label": propValue["label"].split("/")[1]}
             # 9: MULTIPLE (above)
             elif propValue["fmType"] == "12": # Boolean, datetype == xsd:boolean
                 # Fall back to literal (may revisit and suppress)
@@ -732,75 +750,71 @@ class V1RecordToV2:
     def getTransformedByTypeCount(self):
         return self.__transformedByTypeCount
                 
-def cstopsInRecord(record, propPath=""):
-
-    stopped = []
-
-    for prop in record:
-
-        if prop in ["uri"]:
-            continue
-
-        propValue = record[prop]            
-
-        if propValue["type"] == "cnodes": # fmType == "9"
-            if "stopped" in propValue:
-                stopped.append(prop if propPath == "" else propPath + "/" + prop)
-                continue
-            for i, srecord in enumerate(propValue["value"]):
-                propI = "{}[{}]".format(prop, srecord["uri"]["value"].split("-")[-1]) # ie/ record latest IEN
-                propIPath = propI if not propPath else propPath + "/" + propI
-                stopped.extend(cstopsInRecord(srecord, propIPath))
-                            
-    return stopped
-    
 """
 Recache any stopped resources - exceptions if can't - and restore in a rebuilt
 reply.
+
+This ONLY works if DESCRIBING one at a time (top level) can safely get a broad node. From v3, see 'fmqlContainsTotal' (total number of contained elements seen apart from stopping) as well as 'fmqlMaxStopped' at the global level of a Reply
 """
-def recacheForCSTOPs(stationNo, typId, fmqlIF, flushToRF=True):
+def recacheForCSTOPs(stationNo, typId, fmqlIF, flushToRF=False):
 
     start = datetime.now()
     logLocn = re.sub(r'Data\/', 'Logs/', DATA_LOCN_TEMPL).format(stationNo)
     configLogging(stationNo, "REMOVECSTOP")
     logging.info("Begin removal of cstops from {}".format(typId))
     dataLocn = DATA_LOCN_TEMPL.format(stationNo)
-    dataReplyStore = FMQLReplyStore(dataLocn, isOrdered=True)  
+    dataReplyStore = FMQLReplyStore(dataLocn)  
     # TMP: just to be careful 
     destDataLocn = DATARF_LOCN_TEMPL.format(stationNo) if flushToRF else dataLocn
-    rfReplyStore = FMQLReplyStore(destDataLocn, isOrdered=True, useZip=True)     
+    rfReplyStore = FMQLReplyStore(destDataLocn, useZip=True)     
     replyIter = dataReplyStore.iterator(onlyTypes=[typId])
     noRecachedStoppedResources = 0
     noRepliesWithStoppedResources = 0
-    v2FromV1 = V1RecordToV2(forMongo=True)
-    for i, reply in enumerate(next(replyIter), 1):
+    # Too hard to have to burrow to find - instead embedded stopped resource path up top
+    def fmqlStopped(resource, level=0):
+        if "fmqlStopped" in resource:   
+            return level+1
+        for prop in resource:
+            if isinstance(resource[prop], list):
+                for sresource in resource[prop]:
+                    slevel = fmqlStopped(sresource, level+1)
+                    if slevel:
+                        return slevel
+    for i, reply in enumerate(replyIter, 1):
         replyFile = replyIter.currentReplyFile()
+        if not re.search(r'\-S', replyFile):
+            logging.info("Leaving {} as had NO stops".format(replyFile))
+            continue # relies on Marker
         replyHasStoppedResources = False
         nresults = []
         for resource in reply["results"]:
-            if "fmqlStopped" in resource:
+            level = fmqlStopped(resource)
+            if level:
                 replyHasStoppedResources = True
-                logging.info("Encountered stopped resource {} in {} - recaching individually".format(resource["_id"], replyFile))
+                logging.info("Encountered stopped resource {} at level {} in {} - recaching individually".format(resource["_id"], level, replyFile))
                 query = "DESCRIBE {} CSTOP {}".format(resource["_id"], 1000000)
                 queryStart = datetime.now()
-                v1ResourceReply = fmqlIF.invokeQuery(query)
+                resourceReply = fmqlIF.invokeQuery(query)
                 logging.info("Re-cache took {}".format(str(datetime.now() - queryStart)))
-                if "stopped" in v1ResourceReply: # should come in in new FMQL ie/ top level BUT older not -- hence fmqlStopped check in v2 form.
+                if "fmqlMaxStopped" in resourceReply: 
                     raise Exception("Re-query Stopped too {} - exiting".format(resource["_id"])) 
-                nresource = v2FromV1.transform(v1ResourceReply["results"][0])
-                if "fmqlStopped" in nresource:
-                    raise Exception("Re-query stopped too {} - exiting".format(nresource["_id"]))
+                nresource = resourceReply["results"][0]
+                if "fmqlStopped" in nresource: # "legacy" check
+                    raise Exception("Internal V3 Error: shouldn't happen as fmqlMaxStopped should stop it")
                 noRecachedStoppedResources += 1
                 nresults.append(nresource)
             else:
                 nresults.append(resource)
-        if replyHasStoppedResources:
-            reply["results"] = nresults
-            noRepliesWithStoppedResources += 1
-            rfReplyStore.flush(reply)
-            logging.info("Reflushing {} as original had stops".format(replyFile))
-        else:
-            logging.info("Leaving {} as original had NO stops".format(replyFile))
+        if not replyHasStoppedResources:
+            raise Exception("V3 Internal Error: reply file marked with an -S but no stopped resources")
+        reply["results"] = nresults
+        del reply["fmqlMaxStopped"] # no need any more
+        noRepliesWithStoppedResources += 1
+        if not flushToRF: # only done if not flushing to RF
+            logging.info("Deleting obsolete stopped reply file {}".format(replyFile))
+            os.remove("{}{}".format(dataLocn, replyFile))
+        logging.info("Reflushing {} w/o stops".format(re.sub(r'\-S', '', replyFile)))
+        rfReplyStore.flush(reply)
     finish = datetime.now()
     if noRepliesWithStoppedResources:
         logging.info("FINISHED at {} in {}: recached {:,} resources from {:,} replies after seeing {:,} replies".format(
@@ -821,8 +835,27 @@ def recacheForCSTOPs(stationNo, typId, fmqlIF, flushToRF=True):
 metaOfVistA for cutTime and key KSP ids including name - requires 8989_3 and 
 PostMaster user needs to be cached first.
 
+Naming:
+- name | KSP domain name (first part of URL)
+- longName | KSP default institution name
+and notes (as expected), default institution IEN matches station number 
+
 Note: older version used last 3.081, 409.84 too and they could be added
 optionally later and from Data location.
+
+TODO:
+- KSP/production (PROD^XUPROD - prod info in KSP)
+  - https://github.com/caregraf/fmqlutils/issues/4 ... about Entity extractors built in now to run to check my results with those
+- Last 3.081,
+- Build etc DESCRIBE LAST
+- Last Exp DESCRIBE LAST 757_01 
+- Parameters: any global settings worth (can be) taken? 
+  ... S SYS=$S($D(VPRSYS):VPRSYS,1:$$GET^XPAR("SYS","VPR SYSTEM NAME"))
+and
+> Every Fileman installation has a SITE name and number. These values are
+established when initializing Fileman, and are stored in the ^DD("SITE") and
+^DD("SITE",1) nodes.
+... need to go back to a DESCRIBE SYSTEM?
 """
 def metaOfVistA(stationNo):
 
@@ -837,10 +870,12 @@ def metaOfVistA(stationNo):
     fmqlReplyStore = FMQLReplyStore(DATAMETA_LOCN_TEMPL.format(stationNo))
     kspReply = fmqlReplyStore.lastReplyOfType("8989_3")
     postMasterReply = fmqlReplyStore.lastReplyOfType("200")
-    if not (kspReply and postMasterReply):
+    installReply = fmqlReplyStore.lastReplyOfType("9_7")
+    if not (kspReply and postMasterReply and installReply):
         raise Exception("Haven't run fmqlcacher ... META so can't see metaInfo")
     kspResource = kspReply["results"][0]
     postMasterResource = postMasterReply["results"][0]
+    firstInstallResource = installReply["results"][0]
 
     metaInfo = {"stationNo": stationNo}    
     metaInfo["longName"] = kspResource["default_institution"]["label"]
@@ -851,10 +886,27 @@ def metaOfVistA(stationNo):
         metaInfo["name"] = re.sub(r'-', ' ', dnlMatch.group(1)).title()
     defaultInstitutionIEN = kspResource["default_institution"]["id"].split("-")[1]
     if defaultInstitutionIEN == stationNo:
-        metaInfo["isSNODefaultInstitutionIEN"] = True
+        metaInfo["isSNODefaultInstitutionIEN"] = True # ie/ 668 Spokane etc and name Spokane taken for long_name
+    else: # ex/ Anc, Alaska
+        metaInfo["defaultInstitutionIEN"] = defaultInstitutionIEN
+    if "dns_ip" in kspResource:
+        metaInfo["dnsIP"] = kspResource["dns_ip"]
+    if "production" in kspResource:
+        metaInfo["isProduction"] = kspResource["production"]
+    for pprop in [("facility_iso", "iso"), ("facility_cio", "cio")]:
+        if pprop[0] in kspResource:
+            metaInfo[pprop[1]] = kspResource[pprop[0]]
 
+    if "date_entered" in postMasterResource:
+        metaInfo["dateEnteredPostMaster"] = postMasterResource["date_entered"]["value"]
     metaInfo["lastSignonPostMaster"]  = postMasterResource["last_signon_date_time"]["value"]
     metaInfo["cutDate"] = metaInfo["lastSignonPostMaster"].split("T")[0]
+    
+    # probably KERNEL - VIRGIN INSTALL 8.0 ... let's see (and may be very common across
+    # VistA's.
+    metaInfo["nameFirstInstall"] = firstInstallResource["name"] 
+    if "date_loaded" in firstInstallResource:
+        metaInfo["dateLoadedFirstInstall"] = firstInstallResource["date_loaded"]["value"]
         
     json.dump(metaInfo, open(jsnFl, "w"), indent=4)
                 
@@ -945,16 +997,16 @@ Ex/ 38.1 holds lot's of access logs 38.11. Some lead to CSTOPs and need
 to be recached. Flip will end up with 38.11-less 38.1's and distinct, 5000
 clumped 38.11's in DataRF
 """
-def flip(stationNo, typId, multipleProp, multipleTypId, multipleTypName, explicitInheritProps=[], mlimit=7500, fmqlIF=None, useRF=False):
+def flip(stationNo, typId, multipleProp, multipleTypId, multipleTypName, explicitInheritProps=[], mlimit=7500, useRF=False):
 
     startFlip = datetime.now()
     logLocn = re.sub(r'Data\/', 'Logs/', DATA_LOCN_TEMPL).format(stationNo)
     configLogging(stationNo, "FLIP")
     logging.info("Begin flipping {} of {}".format(typId, multipleProp))
     sourceDataLocn = DATARF_LOCN_TEMPL.format(stationNo) if useRF else DATA_LOCN_TEMPL.format(stationNo)
-    dataReplyStore = FMQLReplyStore(sourceDataLocn, isOrdered=True)        
+    dataReplyStore = FMQLReplyStore(sourceDataLocn)        
     destDataLocn = DATARF_LOCN_TEMPL.format(stationNo)
-    rfReplyStore = FMQLReplyStore(destDataLocn, isOrdered=True, useZip=True)
+    rfReplyStore = FMQLReplyStore(destDataLocn, useZip=True)
     replyIter = dataReplyStore.iterator(onlyTypes=[typId])
     includeParentLabel = True if typId not in PATIENT_NAME_TYPES else False
     mIEN = 0 # Note: only works if walk all multiples so each gets unique no
@@ -964,13 +1016,13 @@ def flip(stationNo, typId, multipleProp, multipleTypId, multipleTypName, explici
     ptUpProp = "{}_container_".format(multipleProp)
     cntMultipleProp = "{}_count_".format(multipleProp) # adding in counter to parent for mult
     noResourcesWOMultiple = 0
-    for i, reply in enumerate(next(replyIter), 1):
+    for i, reply in enumerate(replyIter, 1):
         replyFile = replyIter.currentReplyFile()
         reply["lessResults"] = []
         for resource in reply["results"]:
             if multipleProp not in resource:
                 if "fmqlStopped" in resource and multipleProp in resource["fmqlStopped"]:
-                    raise Exception("Didn't expect to run against stopped for multiple {} but resource {} stopped".format(multipleProp, resource["_id"]))
+                    raise Exception("Didn't expect to run against stopped for multiple {} but resource {} of {} stopped".format(multipleProp, resource["_id"], replyFile))
                 noResourcesWOMultiple += 1
                 reply["lessResults"].append(resource) # still add
                 continue # legitimate (unusual but ...)
@@ -981,7 +1033,7 @@ def flip(stationNo, typId, multipleProp, multipleTypId, multipleTypName, explici
                 if ptUpProp in mresource:
                     raise Exception("'{}' in source data as a property - reserved".format(ptUpProp))
                 if "fmqlStopped" in mresource: # simple for now
-                    raise Exception("Don't expect STOP within multiple")
+                    raise Exception("Don't expect STOP within multiple of {}".format(replyFile))
                 mIEN += 1 # doing simple IENs from complete walk 
                 mId = "{}-{}".format(multipleTypId, mIEN)
                 mresource["_id"] = mId
@@ -1091,7 +1143,7 @@ def main():
     replyStore = FMQLReplyStore("/data/vista/999/Data/")
     flTyp = "80"
     iter = replyStore.iterator(onlyTypes=["80"])
-    for flJSON in next(iter):
+    for flJSON in iter:
         print("First {}".format(len(flJSON["results"])))
         break
     lastReplyOfType = replyStore.lastReplyOfType("80")
