@@ -1,13 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: utf8 -*-
 
+# (c) 2015-2020 caregraf
+
 import os
 import sys
 import json
 import shutil
 from datetime import datetime
 import re
-from collections import defaultdict, OrderedDict
+from collections import defaultdict
 import logging
 
 from .cacherUtils import FMQLReplyStore, V1RecordToV2, ensureCacheLocations, DATAV1_LOCN_TEMPL, DATA_LOCN_TEMPL, LOG_LOCN_TEMPL
@@ -18,9 +20,6 @@ Turn FMQL v1.* form replies to v2 form
   - REM: v1 is zip or json; v2 can be either, set with 'useZip'
 - will restart where it left off or rebuild from scratch
 - notes any transformation errors (shouldn't happen)
-
-TODO: once flipper in, allow to apply flipper AFTER doing transform basic
-ie/ pipeline
 """
 def toV2Data(stationNumber, onlyTypes=None, forceRedo=False, useZip=True, forMongo=True):
 
@@ -36,7 +35,7 @@ def toV2Data(stationNumber, onlyTypes=None, forceRedo=False, useZip=True, forMon
     systemId = stationNumber
     logging.info("V2ing Data of VistA {} [PID {}]".format(systemId, os.getpid()))
     
-    v1ReplyStore = FMQLReplyStore(DATAV1_LOCN_TEMPL.format(stationNumber), isOrdered=True)
+    v1ReplyStore = FMQLReplyStore(DATAV1_LOCN_TEMPL.format(stationNumber))
     logging.info("Started Reply Cache V1 - uses {} - total replies {}".format("ZIP" if v1ReplyStore.usesZip() else "JSON", v1ReplyStore.totalReplies()))
         
     v2ReplyStore = FMQLReplyStore(DATA_LOCN_TEMPL.format(stationNumber), useZip=useZip)
@@ -60,9 +59,9 @@ def toV2Data(stationNumber, onlyTypes=None, forceRedo=False, useZip=True, forMon
     transformed = 0
     transformedRecords = 0
     startT = datetime.now() # start before iteration
-    for reply in next(replyIterator):
+    for reply in replyIterator:
         
-        nreply = OrderedDict([("results", [])])
+        nreply = {"results": []}
         for nrProp in reply:
             if nrProp in ["count", "results"]:
                 continue
@@ -100,47 +99,23 @@ def toV2Data(stationNumber, onlyTypes=None, forceRedo=False, useZip=True, forMon
 
 # ############################# Driver ####################################
 
-"""
-Form of configs
-
-    {
-        "stationNumber": "999",
-        "included": ["2"],
-        "forceRedo": False,
-        "forMongo": True
-    }
-    
-"""
 def main():
 
-    assert sys.version_info >= (3, 1)
-
-    if len(sys.argv) < 2:
-        print("need to specify configuration ex/ {VISTAAB}.json - exiting")
-        return
-
-    configBase = sys.argv[1].split(".")[0]
-
-    if not os.path.isfile("{}.json".format(configBase)):
-        print("No config file {}.json - exiting".format(configBase))
-        return
-
-    config = json.load(open("{}.json".format(configBase)), object_pairs_hook=OrderedDict)
-
-    if "stationNumber" not in config:
-        print("No station number in configuration - exiting")
-        return
-    stationNumber = config["stationNumber"]
-
-    if "included" not in config or len(config["included"]) == 0:
-        types = None
-    else:
-        types = config["included"]
-        
-    forceRedo = True if "forceRedo" in config and config["forceRedo"] == True else False
-    forMongo = False if "forMongo" in config and config["forMongo"] == False else True
+    assert sys.version_info >= (3, 4)
     
-    toV2Data(stationNumber, onlyTypes=types, forceRedo=forceRedo, forMongo=forMongo)
+    try:
+        stationNumber = sys.argv[1]
+        typ = sys.argv[2]
+    except IndexError:
+        raise SystemExit("Usage _EXE_ STATIONNO TYP [FORCEREDO=NO|YES]")
+
+    forceRedo = True
+    if len(sys.argv) > 3:
+        forceRedoArg = sys.argv[3] 
+        if forceRedoArg.lower() == "no":
+            forceRedo = False
+    
+    toV2Data(stationNumber, onlyTypes=[typ], forceRedo=forceRedo, forMongo=True)
 
 if __name__ == "__main__":
     main()
