@@ -1,6 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf8 -*-
 
+# (c) 2017-2020 caregraf
+
 import os
 import sys
 import re
@@ -17,10 +19,6 @@ Using data, defines a type according to the presence and (selectively) the frequ
 REM: VistA typing/files/collections n enums etc often too broad, too narrow or muddy.
 (Sub)Typing pictures VistA but often needs better categorization/groupings, things
 refined from reframing the data first and this utility does allow for that.
-
-TODO: 
-- in general: see one use of u' below. Consider that everywhere OR from __future__ import unicode_literals but the latter may need stricters formats (442:8927/items that tries this)
-- see below: separation of reporter/jsonschema etc [will move reporter into reporter module and go to making simple site] generation from type info
   
 Arguments:
 - subTypeProps: one or more properties used to subtype by. Note that
@@ -43,7 +41,7 @@ DEFAULT_MAX_NOMINAL_VALUES = 60 # maxNominalValues=60 as 55 is # dialogs of 100
 DEFAULT_FORCE_COUNT_PTER_TYPES = ["4", "44", "200"] # user, location, patient
 USER_NAME_TYPES = ["220_5"]
 DEFAULT_FORCE_COUNT_PTER_TYPES.extend(USER_NAME_TYPES)
-PATIENT_NAME_TYPES = ["2", "9000001", "55", "63", "70", "631", "665", "690", "790", "38_1"] # mainly dinumed, 2005 as its label has id info
+PATIENT_NAME_TYPES = ["2", "9000001", "55", "63", "70", "631", "665", "690", "790", "38_1", "354_7"] # mainly dinumed, 2005 as its label has id info
 DEFAULT_FORCE_COUNT_PTER_TYPES.extend(PATIENT_NAME_TYPES)
 
 class SubTypeReducer():
@@ -89,7 +87,8 @@ class SubTypeReducer():
                     if "value" in resource[stp]: # will only split by year
                         stpv = resource[stp]["value"].split("-")[0]
                     elif "label" in resource[stp]:
-                        stpv = "{} [{}]".format(resource[stp]["label"].encode("ascii", "ignore"), resource[stp]["id"])
+                        # PY2 -> 3: removed .encode("ascii", "ignore") for label
+                        stpv = "{} [{}]".format(resource[stp]["label"], resource[stp]["id"])
                     else: # should never happen
                         raise Exception("Expect structured value of sub type property {} to be a POINTER or DATE (Year)".format(stp))
                 elif isinstance(resource[stp], bool):
@@ -109,9 +108,9 @@ class SubTypeReducer():
     def reductions(self):
         results = []
         allReduction = self.__allTypeReducer.reduction()
+        allReduction["_reductionEndDay"] = datetime.now().strftime("%Y-%m-%d")
         results.append(allReduction)
         allTotal = allReduction["_total"]
-        allTotal["_reductionEndDay"] = datetime.now().strftime("%Y-%m-%d")
         for subTypeId in self.__typeReducers:
             reduction = self.__typeReducers[subTypeId].reduction()
             reduction["_subTypeId"] = subTypeId
@@ -172,9 +171,9 @@ class TypeReducer():
     def __reduce(self, resource, typeData, parentPropPath=""):
 
         def isPointer(value):
-            if len(value.keys()) == 0:
+            if len(value) == 0:
                 return False # shouldn't happen but some synthesized has {} value
-            if len(set(value.keys()) - set(["id", "sameAs", "sameAsLabel", "label"])):
+            if len(set(value) - set(["id", "sameAs", "sameAsLabel", "label"])):
                 return False
             return True
         
@@ -221,7 +220,7 @@ class TypeReducer():
                 return
             # First exclusion: if long then a real literal and not just a value 
             # (NOTE: bool's fall in here too)
-            if isinstance(litValue, basestring) and qprop not in self.__forceCountProperties and len(litValue.split(" ")) > 5:
+            if isinstance(litValue, str) and qprop not in self.__forceCountProperties and len(litValue.split(" ")) > 5:
                 self.__dontCountLiteralProp.add(qprop)
                 if "byValueCount" in propInfo:
                     del propInfo["byValueCount"]
@@ -297,17 +296,28 @@ class TypeReducer():
                 return
             propInfo["byValueCount"][len(listValue)] = 1
 
+        """
+        Compromise that gives data for graphing, trending
+        time including weekday/weekend splits and per day
+        creation graphs but which doesn't collect too much 
+        for types with many dates.
+        """
         def countDateValue(prop, propInfo, dtValue):
             dayValue = dtValue.split("T")[0]
             dayDT = validDayDT(dayValue)
             if not dayDT:
                 return
             dtps = dayValue.split("-")
+            monthValue = "{}-{}".format(dtps[0], dtps[1])
             # Not doing WEEK as WEEK in what year?
             #     dtToCount = dayDT.isocalendar()[1]                
             if self.__countDateType == "MONTH":
-                dtToCount = "{}-{}".format(dtps[0], dtps[1])
+                dtToCount = monthValue
             elif self.__countDateType == "DAY":
+                # May downstep all but createProp no matter what
+                # if self.__createDateProp != prop:
+                #    dtToCount = monthValue
+                # else:
                 dtToCount = dayValue
             else: # "YEAR" is default
                 dtToCount = dtps[0]            
@@ -324,7 +334,7 @@ class TypeReducer():
                         
         typeData["_total"] += 1
         
-        if "_firstIEN" not in typeData and len(set(resource.keys()).intersection(["_id", "id"])):
+        if "_firstIEN" not in typeData and len(set(resource).intersection(["_id", "id"])):
             idProp = "id" if "id" in resource else "_id"
             self.__typeData["_firstIEN"] = resource[idProp].split("-")[1]
                     
@@ -341,7 +351,7 @@ class TypeReducer():
                     
         createDate = getCreateDate(resource, self.__createDateProp)
         dateValues = []
-        for prop in [key for key in resource.keys() if key not in ["id", "_id", "type", "label", "ien", "fmqlStopped"]]: 
+        for prop in [key for key in resource if key not in ["id", "_id", "type", "label", "ien", "fmqlStopped"]]: 
 
             value = resource[prop] 
 
@@ -388,7 +398,7 @@ class TypeReducer():
                     typeData[prop]["rangeTypes"].append(rangeType)
                 ptrType = value["id"].split("-")[0]
                 if "label" in value and ptrType not in self.__deidentifyPointerTypes:
-                    ptrValue = u'{} [{}]'.format(value["label"], value["id"])
+                    ptrValue = '{} [{}]'.format(value["label"], value["id"])
                 else:
                     ptrValue = value["id"]
                 countByPointerValue(qprop, typeData[prop], ptrValue)
