@@ -1,17 +1,22 @@
 #!/usr/bin/env python
 # -*- coding: utf8 -*-
 
+# (c) 2017-2020 caregraf
+
 import os
 import sys
 import re
 import json
-from collections import OrderedDict, defaultdict, Counter
+from collections import defaultdict, Counter
 from datetime import datetime
+from dateutil.relativedelta import relativedelta
 import logging
 
-from ..cacher.cacherUtils import DATA_LOCN_TEMPL, DATARF_LOCN_TEMPL
-from reduceReportTypes import TYPER_REDUCTIONS_LOCN_TEMPL
-from reduceReportType import DEFAULT_MAX_NOMINAL_VALUES
+from .. import VISTA_DATA_BASE_DIR
+from ..cacher.cacherUtils import FMQLReplyStore, FilteredResultIterator
+from .reduceReportType import TYPER_REDUCTIONS_LOCN_TEMPL
+from .reduceType import DEFAULT_MAX_NOMINAL_VALUES
+from ..reporter.reportUtils import muBVC
 
 """
 Expect 1 ALL (returned first) and many sub's with one or more properties. Note
@@ -74,7 +79,7 @@ def combineSubTypes(sts, subTypePropsSubset=[], forceCountProps=[], forceCountAl
     csts = []
     for stIdWanted in sorted(stsBySTIdWanted):
         # only prop kept  
-        cst = OrderedDict([("_subTypeId", stIdWanted), ("_total", 0)])
+        cst = {"_subTypeId": stIdWanted, "_total": 0}
         if createDateProp:
             cst["_createDateProp"] = createDateProp
         csts.append(cst)
@@ -115,27 +120,38 @@ def combineSubTypes(sts, subTypePropsSubset=[], forceCountProps=[], forceCountAl
     
 """
 Check Data: array of fileType and what's needed (array)
-... ALL | TYPE | {REDUCTIONLABEL}
+... ALL(RF?) | TYPE | {REDUCTIONLABEL}
+... stoppedToo can be asserted too to allow -S as well
 
-Ex/ [{"fileType": "3_081", "check": "TYPE"}]
+Ex/ [
+        {"fileType": "3_081", "check": "TYPE"},
+        
+        {"fileType": "123_5", "check": "ALL", "stoppedToo": True}
+    ]
 """
 def checkDataPresent(stationNumber, dataToCheck):
-    hasAll = True
+    hasAll = True # all there (assume so but correct below)
     for dataInfo in dataToCheck:
         ftyp = dataInfo["fileType"]
         if not isinstance(dataInfo["check"], list):
             dataInfo["check"] = [dataInfo["check"]]
         checked = {}
+        stoppedToo = dataInfo.get("stoppedToo", False)
         for toCheck in dataInfo["check"]:
             if re.match("ALL", toCheck):
                 if toCheck == "ALL":
                     # note: doesn't check completeness. One will do!
-                    fl = DATA_LOCN_TEMPL.format(stationNumber) + "{}-0.zip".format(ftyp)
+                    fl = "{}{}/Data/{}".format(VISTA_DATA_BASE_DIR, stationNumber, "{}-0.zip".format(ftyp))
                 elif toCheck == "ALLRF":
-                    fl = DATARF_LOCN_TEMPL.format(stationNumber) + "{}-0.zip".format(ftyp)
+                    fl = "{}{}/DataRF/{}".format(VISTA_DATA_BASE_DIR, stationNumber, "{}-0.zip".format(ftyp))
                 else:
                     raise Exception("ALL options are ALL|ALLRF")
                 if not os.path.isfile(fl):
+                    if stoppedToo:
+                        fl = re.sub(r'\-0', '-S-0', fl)
+                        if os.path.isfile(fl):
+                            checked[toCheck] = True
+                            continue
                     checked[toCheck] = False
                     hasAll = False
                 else:
@@ -162,9 +178,9 @@ def singleValue(typeRed, prop, ifMissingValue=""):
         if ifMissingValue:
             return ifMissingValue
         raise Exception("Unexpected missing property {} of reduction".format(prop))
-    if not ("byValueCount" in typeRed[prop] and len(typeRed[prop]["byValueCount"].keys()) == 1):
+    if not ("byValueCount" in typeRed[prop] and len(typeRed[prop]["byValueCount"]) == 1):
         raise Exception("Unexpected > 1 value for prop {} of reduction".format(prop))
-    return typeRed[prop]["byValueCount"].keys()[0]
+    return list(typeRed[prop]["byValueCount"])[0]
     
 """
 mu a bvc
@@ -172,30 +188,6 @@ mu a bvc
 Note: forceShowCount is to allow an ST BVC use to force a count for
 a single value if count < total
 """
-def muBVC(bvc, separator=", ", forceShowCount=False, countOnlyIfOver=-1, totalOver=-1):
-    def labelKey(k):
-        if re.search(r' \[', k):
-            return k.split(" [")[0]
-        if not re.search(r':', k):
-            return k
-        return k.split(":")[1]
-    if len(bvc) == 1:
-        k = bvc.keys()[0]
-        if not forceShowCount:
-            return labelKey(k)
-        return "{} [{:,}]".format(labelKey(k), bvc[k]) 
-    if countOnlyIfOver != -1 and len(bvc) > countOnlyIfOver:
-        return "{:,}".format(len(bvc))
-    if totalOver != -1 and len(list(bvc)) > totalOver:
-        sBVs = sorted(list(bvc), key=lambda x: bvc[x], reverse=True)
-        return "{} and {:,} Others [{:,}]".format(
-            separator.join(["{} [{:,}]".format(labelKey(k), bvc[k]) for k in sBVs[0:totalOver]]),
-            len(sBVs[totalOver:]),
-            sum(bvc[k] for k in sBVs[totalOver:])
-        )
-    mu = separator.join(["{} [{:,}]".format(labelKey(k), bvc[k]) for k in sorted(bvc, key=lambda x: bvc[x], reverse=True)])
-    return mu
-    
 def muBVCOfSTProp(st, prop, separator=", ", addNotSet=True):
     if prop not in st:
         return "" # don't say NOTSET - just leave blank
@@ -220,4 +212,115 @@ def thresholdProps(typeData):
         byThreshold[thresholds[currentThresholdIndex]].append(prop)
     return byThreshold
 
+# ex/ refsOfST(st, "200") <=> userRefsOfST
+def refsOfST(st, typ):
+    refs = set()
+    for prop in st:
+        if re.match(r'\_', prop):
+            continue
+        if st[prop]["type"] == "LIST":
+            refs |= refsOfST(st[prop]["reduction"], typ)
+        if "rangeTypes" not in st[prop]:
+            continue
+        if typ not in st[prop]["rangeTypes"]:
+            continue
+        if "byValueCount" not in st[prop]:
+            raise Exception("By Value Count not supported for {}/{}".format(st["_subTypeId"], prop))
+        for ref in st[prop]["byValueCount"]:
+            if not re.search(f'{typ}\-', ref):
+                continue
+            refs.add(ref)
+    return refs 
+
+"""
+Time Series CSV for a type, using a day prop.
+
+You can use upToDay (typically cutDate) but ONLY when dayProp is the type's creation property. timeBack is of form:
+    {"months": #} or {"days": #} or {"years": #} or a combination ie/ > 1 arg
+
+Specifically, header row and array of data rows as single string values
+    "Date,ColId1,ColId2..."
+    "2020-...,#,#"
+    ...
+where ColId1 ... breakdown by the Splitter. If there is no Splitter then
+you see
+    "Date,ALL"
     
+Ex/ Splitter
+
+    class Splitter:
+        def __init__(self, stationNo):
+            self.__stationNo = stationNo
+        def split(self, resource):
+            if ...
+                return "" # suppress
+            ... categorize
+            return colId
+    
+Splitter can also SUPPRESS entries (ex/ if walking visits and only want encounters,
+then can suppress the irrelevant)
+    
+Returns rows
+
+Fits into a Pandas DF
+    df = pd.read_csv(
+        io.StringIO("\n".join(rows)),
+        index_col=0, 
+        parse_dates=True
+    )
+(not returning as easier to just serialize string for now)
+    
+To get colIds for display:
+    rows[0].split(",")[1:]
+"""        
+def makeTypeTSCSVByDay(stationNo, typId, dayProp, upToDay="", timeBack=None, splitter=None, efficientWalked=True):
+
+    dataLocn = "{}{}/Data/".format(VISTA_DATA_BASE_DIR, stationNo) 
+    startAtReply = ""
+    if upToDay and timeBack:
+        upToDayDT = datetime.strptime(upToDay.split("T")[0], "%Y-%m-%d")
+        # timeBack == {"years": #} etc.
+        onAndAfterDayDT = upToDayDT - relativedelta(**timeBack)
+        onAndAfterDay = datetime.strftime(onAndAfterDayDT, "%Y-%m-%d")
+        if efficientWalked:
+            store = FMQLReplyStore(dataLocn)
+            startAtReply = store.firstReplyFileOnOrAfterCreateDay(typId, dayProp, onAndAfterDay)
+            print(f'TS CSV: iterating {typId} starting with reply {startAtReply} from {onAndAfterDay} to {upToDay} for creation property {dayProp}')
+        else:
+            print(f'TS CSV: iterating {typId} from the beginning (not efficient walk) from {onAndAfterDay} to {upToDay} for creation property {dayProp}')           
+    else:
+        print(f'TS CSV: iterating all of {typId} w/o time window')
+    resourceIter = FilteredResultIterator(dataLocn, typId, startAtReply=startAtReply)
+    dayCounts = defaultdict(lambda: Counter())
+    colIds = set()
+    for resource in resourceIter:
+        if dayProp not in resource:
+            print(f'** Warning: missing {dayProp} from resource {resource["_id"]} - skipping')
+            continue
+        dt = resource[dayProp]["value"].split("T")[0]
+        if upToDay:
+            if dt < onAndAfterDay:
+                continue
+            if dt >= upToDay:
+                if efficientWalked:
+                    break # assuming order
+                else:
+                    continue
+        if splitter:
+            colId = splitter.split(resource)
+            if colId == "": # can suppress
+                continue
+            # , in colId messes things 653/648 had issue (ex/ VANCOUVER,WA DIV ...) 
+            colId = re.sub(r'\,', ' ', colId) 
+            colIds.add(colId)
+        dayCounts[dt][colId] += 1
+    colIds = sorted(list(colIds), key=lambda x: sum(dayCounts[dt][x] for dt in dayCounts if x in dayCounts[dt]), reverse=True) if len(colIds) else ["ALL"]
+    rows = [f'Date,{",".join(colIds)}']
+    for dt in sorted(dayCounts):
+        row = [dt]
+        for colId in colIds:
+            cnt = 0 if colId not in dayCounts[dt] else dayCounts[dt][colId]
+            row.append(str(cnt))
+        rows.append(",".join(row))
+    print(f'Returning TS with {len(colIds)} criteria over {len(rows)-1} dates from {rows[1].split(",")[0]} to {rows[-1].split(",")[0]}')
+    return rows
